Type: Package
Package: autoFC
Title: Automatic Toolkit for Construction, Optimization, Scoring and
        Simulation of Forced-Choice Tests
Version: 1.0.0.1100
Authors@R: c(person("Mengtong", "Li", email = "mt_li@fudan.edu.cn", role = c("cre", "aut"), comment = c(ORCID = "0000-0002-1766-4976")), person("Tianjun", "Sun", email = "tsun5@illinois.edu", role = "aut", comment = c(ORCID = "0000-0002-3655-0042")), person("Bo", "Zhang", email = "bozhang3065@gmail.com", role = "aut", comment = c(ORCID = "0000-0002-6730-7336")))
Description: Forced-choice (FC) response has gained increasing popularity
    and interest for its resistance to faking when well-designed (Cao &
    Drasgow, 2019 <doi:10.1037/apl0000414>). To established well-designed
    FC scales, typically each item within a block should measure different
    trait and have similar level of social desirability (Zhang et al.,
    2020 <doi:10.1177/1094428119836486>). Recent study also suggests the
    importance of high inter-item agreement of social desirability between
    items within a block (Pavlov et al., 2021
    <doi:10.31234/osf.io/hmnrc>).  In addition to this, FC developers may
    also need to maximize factor loading differences (Brown &
    Maydeu-Olivares, 2011 <doi:10.1177/0013164410375112>) or minimize item
    location differences (Cao & Drasgow, 2019 <doi:10.1037/apl0000414>)
    depending on scoring models. Decision of which items should be
    assigned to the same block, also called as item pairing, is thus critical 
    to the quality of an FC test. Because such pairing process often requires
    researchers to meet multiple objectives, manual pairing becomes impractical 
    or even not feasible once the number of latent traits and/or number of items 
    per elevates. To address these problems, autoFC is developed as a
    automatic and efficient tool for facilitating the automatic construction of 
    FC tests (Li et al., 2022 <doi:10.1177/01466216211051726>), essentially
    exempting users from the burden of manual item pairing.
    Given characteristics of each item (and item responses), FC measures
    can be constructed either automatically based on user-defined pairing
    criteria and weights, or based on exact specifications of each block
    (i.e., blueprint; see Li et al., 2025
    <doi:10.1177/10944281241229784>). Users can also generate simulated
    responses based on the Thurstonian Item Response Theory model (Brown &
    Maydeu-Olivares, 2011 <doi:10.1177/0013164410375112>) and predict
    trait scores of simulated/actual respondents based on an estimated
    model.
License: GPL (>= 3)
Depends: R (>= 3.5)
Imports: lavaan, MASS, MplusAutomation, pbapply, rstan, stats
Suggests: knitr, rmarkdown, cmdstanr
Additional_repositories: https://stan-dev.r-universe.dev
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-23 02:58:45 UTC; Mengtong Li
Author: Mengtong Li [cre, aut] (ORCID: <https://orcid.org/0000-0002-1766-4976>),
  Tianjun Sun [aut] (ORCID: <https://orcid.org/0000-0002-3655-0042>),
  Bo Zhang [aut] (ORCID: <https://orcid.org/0000-0002-6730-7336>)
Maintainer: Mengtong Li <mt_li@fudan.edu.cn>
Repository: CRAN
Date/Publication: 2026-08-23 05:10:10 UTC
Built: R 4.5.2; ; 2026-08-23 06:01:28 UTC; unix
